#!/usr/bin/env bash

INSIGHT_SERVER=/insight-server
export JFI_HOME=/opt/jfrog
export JFIS_APP_NAME=${JFIS_APP_NAME:-insight-server}
export JFI_HOME_CORE=${JFI_HOME}/${JFIS_APP_NAME}
export JFIS_LOGS=${JFIS_LOGS:-$JFI_HOME_CORE/logs}
export JFMC_CUSTOM_ENV=${JFMC_CUSTOM_ENV:-"/opt/jfrog/jfmc/entrypoint/setenv.sh"}
export JFMC_HOSTNAME=${JFMC_HOSTNAME:-$(hostname -s)}

logger() {
    DATE_TIME=$(date +"%Y-%m-%d %H:%M:%S")
    if [ -z "$CONTEXT" ]
    then
        CONTEXT=$(caller)
    fi
    MESSAGE=$1
    CONTEXT_LINE=$(echo "$CONTEXT" | awk '{print $1}')
    CONTEXT_FILE=$(echo "$CONTEXT" | awk -F"/" '{print $NF}')
    printf "%s %05s %s %s\n" "$DATE_TIME" "[$CONTEXT_LINE" "$CONTEXT_FILE]" "$MESSAGE"
    CONTEXT=
}

errorExit () {
    echo; logger "ERROR: $1"; echo
    exit 1
}

warn () {
    logger "WARNING: $1"
}

testDirectoryPermissions () {
    local dir_to_check=$1
    local error=false
    local error_type=

    [ -d ${dir_to_check} ] || errorExit "'${dir_to_check}' is not a directory"

    local id_str="'${JFMC_USER_NAME}' (id ${JFMC_USER_ID})"
    # Get current user id to check if its default jfmc uid
    local u_id=$(id -u)
    if [ "${u_id}" != ${JFMC_USER_ID} ]; then
        id_str="id ${u_id}"
    fi

    logger "Testing directory (${dir_to_check}) has read/write permissions for user ${id_str}"

    local test_file=${dir_to_check}/test-permissions

    # Write file
    if echo test > ${test_file} 1> /dev/null 2>&1; then
        # Write succeeded. Testing read...
        if cat ${test_file} > /dev/null; then
            logger "${dir_to_check} has read/write permissions for user ${id_str}"
            rm -f ${test_file}
        else
            error=true
            error_type=readable
        fi
    else
        error=true
        error_type=writable
    fi

    if [ "${error}" == true ]; then
        local stat_data=$(stat -Lc "Directory: %n, permissions: %a, owner: %U, group: %G" ${dir_to_check})
        logger "###########################################################"
        logger "${dir_to_check} is NOT ${error_type}!"
        logger "${stat_data}"
        logger "Mounted directory must be ${error_type} by user ${id_str}"
        logger "###########################################################"
        errorExit "Directory ${dir_to_check} has bad permissions for user ${id_str}"
    fi
}

# Test permissions on JFIS_LOGS
testPermissions () {
    # JFIS_LOGS folder
    testDirectoryPermissions ${JFIS_LOGS}
}

# In case data dirs are missing or not mounted, need to create them
setupDirs () {
    logger "Setting up directories"
    mkdir -p "${JFIS_LOGS}" || errorExit "unable to create following directories : ${JFIS_LOGS}"
}

# Load custom environment and set default values for required variables
# This is added to support SAAS setup
loadCustomEnv() {
    if [ -f "${JFMC_CUSTOM_ENV}" ]; then
        logger "Found a custom environment ${JFMC_CUSTOM_ENV}, loading it ..."
        source ${JFMC_CUSTOM_ENV} || errorExit "Failed to source ${JFMC_CUSTOM_ENV}"

        export JFMC_INSIGHT_SERVER_DB=${JFMC_INSIGHT_SERVER_DB:-"insight_team"}
        export JFIS_ALLOW_IP=${JFIS_ALLOW_IP:-"0.0.0.0/0"}
    fi
}

############### MAIN #####################

setupDirs
testPermissions
loadCustomEnv

JFMC_CONSOLE_LOG="${JFIS_LOGS}"/"${JFIS_APP_NAME}".log
# Write log to console and a file
exec "${INSIGHT_SERVER}" 2>&1 | tee  -a "${JFMC_CONSOLE_LOG}"